c
c***********************************************************************
c  EHOOK
c***********************************************************************
c
      subroutine ehook(mw)                                              0301YC97
c
c     This routine was added on June 2, 1994.  It is the driver for
c     the energy routines in the modular version of POLYRATE.
c
c     This routine optionally accepts the mass-weighted cartesian 
c     coordinates and transforms them to cartesian coordinates (mw=1).
c     The energy is computed using the cartesian coordinates.  If the
c     mass-weighted option was chosen (mw=1), the coordinates are
c     transformed back to mass-weighted at the end.  

c     To add a new potential driver, one just needs to add a new block to 
c     the 'if block' with a call to a routine that accepts cartesian 
c     coordinates (non-mass-weighted) and returns the energy.
c
c     Driver for Energetics
c       The appropriate potential will be called according to the POTNAM 
c       keyword set in the GENERAL section.  This keyword is passed 
c       through the ipot variable:
c
c           ipot               potential
c
c            0                 hooks
c            1                 unit30/unit40
c            2                 ACES
c            3                 unit29
c
c
      implicit double precision (a-h,o-z)
c
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      dimension fmat(n3tm,n3tm)
c
c     Optionally convert the mass-weighted coordinates to cartesian
c
      if (mw.eq.1) call trans(2,n3,amass,x,dx)
c
c     Compute the potential using the appropriate routines
c
      if (ipot.eq.0.and.lgs(30).le.0) then                              1024YC96
c         call surf(v,x,dx,n3tm)
c      else if (ipot.eq.1) then
         call mopeg(v,x,dx,.false.,n3tm)
      else if (ipot.eq.2) then
         call acalc(1,fmat)
      else if (ipot.eq.3) then
      end if
c
c     Put the energy at an appropriate zero of energy
      if (lezer0) v = v-ezer0                     
c
c
c     new format, the bath term is a function of X instead of S
c
      IF (LBATH) THEN
        CALL EFFBATH(0)
      ENDIF
c
c     Optionally transfer back to mass-weighted coordinates
      if (mw.eq.1) call trans(1,n3,amass,x,dx)
c
      return
      end                                           
c
c***********************************************************************
c  GHOOK 
c***********************************************************************
c
      subroutine ghook(mw)                                              0301YC97

c     This routine was added on June 2, 1994.  It is the driver for
c     the first derivative routines in the modular version of POLYRATE.
c
c     This routine optionally accepts the mass-weighted cartesian 
c     coordinates and transforms them to cartesian coordinates (mw=1).
c     The cartesian coordinates are used when computing the energy and
c     first derivatives of the energy with respect to each cartesian 
c     coordinate.  If the mass-weighted option was chosen (mw=1), the 
c     coordinates and first derivatives are transformed back to mass-
c     weighted at the end.  
c
c     To add a new potential driver, one just needs to add a new block to 
c     the 'if block' with a call to a routine that accepts cartesian 
c     coordinates (non-mass-weighted) and returns the energy and first
c     derivatives.
c
c     Driver for Energetics
c       The appropriate potential will be called according to the POTNAM 
c       keyword set in the GENERAL section.  This keyword is passed 
c       through the ipot variable:
c
c           ipot               potential
c
c            0                 hooks
c            1                 unit30/unit40
c            2                 ACES
c            3                 unit29
c
c
      implicit double precision (a-h,o-z)
c
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      dimension fmat(n3tm,n3tm)
c
c     Optionally convert the mass-weighted coordinates to cartesian
c
      if (mw.eq.1) call trans(2,n3,amass,x,dx)
c
c     Compute the potential using the appropriate routines
c
      if (ipot.eq.0.and.lgs(30).le.0) then                              1024YC96
c         call surf(v,x,dx,n3tm)
c      else if (ipot.eq.1) then
         call mopeg(v,x,dx,.true.,n3tm)
      else if (ipot.eq.2) then
         call acalc(2,fmat)
      end if
c
c     Put the energy at an appropriate zero of energy
c
      if (lezer0) v = v-ezer0
c
c
c   calculate the derivative and energy, should be in Cartesian coord
c
      IF (LBATH) THEN
        CALL EFFBATH(1)
      ENDIF
c
c
c     Transfer back to mass-weighted coordinates
      if (mw.eq.1) call trans(1,n3,amass,x,dx)
c
      return
      end                                           
c
c***********************************************************************
c  OHOOK
c***********************************************************************
c
         subroutine ohook (iop)                                         0301YC97
c
c     This routine was rewrittenadded on June 6, 1994.  It is the driver
c     for the geometry optimization in the modular version of POLYRATE.
c
c     This routine optionally accepts a initial guess to the cartesian
c     coordinates of the full system and returns the optimized geometry
c     and energy
c
c     The geometry is optimized for a given chemical species:
c         For IOP = 1, or 2; the chemical species is reactant 1 or 2.
c         For IOP = 3, or 4; the chemical species is product 1 or 2.
c         For IOP = 5, the chemical species is the saddle point.
c
c     To add a new potential driver, one just needs to add a new block to 
c     the 'if block' with a call to a routine that accepts the initial
c     guess cartesians (x) and returns the optimized geometry in the
c     array x and the energy (V).
c
c     Driver for Energetics
c       The appropriate potential will be called according to the POTGEOM 
c       keyword set in the GENERAL section.  This keyword is passed 
c       through the ispot variable:
c
c           igpot              Method
c
c            0                 Polyrate
c            1                 ohook
c            2                 ACES
c
c
      implicit double precision (a-h,o-z)
c
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      dimension fmat(n3tm,n3tm)
c
c     Set-up the geometry arrays for the different potentials.
c
      if (initg(iop).eq.0) then                                         0522PF97
        call gsetp(iop)
      else if (initg(iop).eq.1) then                                    0522PF97
        call gsetm(iop)
      else if (ipot.eq.2) then
        if (iop.eq.5) call gseta
      end if
c
c     Compute the optimized geometry
c
      if (igpot.eq.0) then
        call newt(iop)
      else if (igpot.eq.1) then
        call mopopt(iop)
      else if (igpot.eq.2) then
         call acalc(4,fmat)
      end if
c
c     Put the energy at an appropriate zero of energy
      if (lezer0) v = v-ezer0
c
c   calculate the derivative and energy, should be in Cartesian coord
c
      IF (LBATH) THEN
        CALL EFFBATH(1)
      ENDIF
c
c
      return
      end                                           
c***********************************************************************
c  PREP
c***********************************************************************
c
      subroutine prep
c
c     This routine was added on July 12, 1994.  It is the driver for
c     all preparation routines needed to set up an electronic structure
c     calculation.
c
c     This routine is called after all POLYRATE input has been read in
c     and before the table of input is written to fu6.  It is also
c     before any calculations are done.  This routine should be used
c     to call any electronic structure preparation routines.  No 
c     prep code should be written directly in this subroutine.  Everything
c     must be put in a separate subroutine.

c     To add a new potential driver, one just needs to add a new block to 
c     the 'if block' with a call to the appropriate routine.
c
c     This routine is called if the user specifies that the potential,
c     second derivatives or geometry optimizations are not to be done
c     using an analytic surface with the POLYRATE routines.  The method
c     used is given by the variable ipot, ispot, and/or igpot.
c
c      ipot,ispot,igpot         potential
c
c            0                  analytic PES
c            1                  MOPAC
c            2                  ACES
c            3                  GAMESS
c
c
      implicit double precision (a-h,o-z)
c
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
c
c     Call the appropriate prep routines
c
c      if (ipot .eq. 0) call setup(n3tm)                                 1220WH94
c      if (ipot.eq.1.or.ispot.eq.1.or.igpot.eq.1) then
      if (ipot.eq.0) then
         call mopset
      else if (ipot.eq.2.or.ispot.eq.2.or.igpot.eq.2) then
      else if (ipot.eq.3.or.ispot.eq.3.or.igpot.eq.3) then
      end if
c
      return
      end                                           
c
c***********************************************************************
c  PREPJ
c***********************************************************************
c
      subroutine prepj(jtype)
c
c     This routine was added on May 23, 1995.  It is the species dependent
c     driver for all preparation routines needed to set up an electronic 
c     structure calculation.  Any initializations that are dependent
c     on wheter the reactants, products, saddle point, or generalized
c     transition state is computed should be done here.  JTYPE specifies
c     the species:
c
c           JTYPE               Species
c             1                 Reactant #1
c             2                 Reactant #2
c             3                 Product #1
c             4                 Product #2
c             5                 Saddle point
c             6                 Generalized transition states
c
c     This routine is called after each respective call to stvarj.
c     No prep code should be written directly in this subroutine.
c     Everything must be put in a separate subroutine.
 
c     To add a new potential driver, one just needs to add a new block to
c     the 'if block' with a call to the appropriate routine.
c
c     This routine is called if the user specifies that the potential,
c     second derivatives or geometry optimizations are not to be done
c     using an analytic surface with the POLYRATE routines.  The method
c     used is given by the variable ipot, ispot, and/or igpot.
c
c      ipot,ispot,igpot         potential
c
c            0                  analytic PES
c            1                  MOPAC
c            2                  ACES
c            3                  GAMESS
c
c
      implicit double precision (a-h,o-z)
c
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
c
c     Set the variables that are needed for all potentials
c
      if (jtype.ne.6) then
         call stlgsj(jtype)
         call stvarj(jtype)
      end if
c
c     Call the appropriate prep routines
c
c
      if (ipot.eq.1.or.ispot.eq.1.or.igpot.eq.1) then
      else if (ipot.eq.2.or.ispot.eq.2.or.igpot.eq.2) then
         call aceset(jtype)
      else if (ipot.eq.3.or.ispot.eq.2.or.igpot.eq.2) then
      end if
c
      return
      end
c***********************************************************************
c  HHOOK 
c***********************************************************************
c
      subroutine hhook(xii,fmat,sstp,nend,mw,icent)                     0301YC97
c
c     This routine was added on June 6, 1994.  It is the driver for
c     the second derivative routines in the modular version of POLYRATE.
c
c     This routine optionally accepts the mass-weighted cartesian 
c     coordinates and transforms them to cartesian coordinates (mw=1).
c     The cartesian coordinates are used when computing the energy and
c     second derivatives of the energy with respect to each cartesian 
c     coordinate.  If the mass-weighted option was chosen (mw=1), the 
c     coordinates and first derivatives are transformed back to mass-
c     weighted at the end.  
c
c     To add a new potential driver, one just needs to add a new block to 
c     the 'if block' with a call to a routine that accepts cartesian 
c     coordinates (non-mass-weighted) and returns the Hessian matrix (F)
c
c     Driver for Energetics
c       The appropriate potential will be called according to the POTSECOND 
c       keyword set in the GENERAL section.  This keyword is passed 
c       through the ispot variable:
c
c           ispot              Method
c
c            0                 ghook
c            1                 hhook
c            2                 ACES
c
c
      implicit double precision (a-h,o-z)
c
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
c
      dimension xii(n3tm), fmat(n3tm,n3tm)
c
c      write (6,*) '%%%%%%%%%% calling hhook  %%%%%%%%%%%%%'
c
c     Optionally convert the mass-weighted coordinates to cartesian
c
      if (mw.eq.1) call trans(2,n3,amass,x,dx)
c
c     Compute the potential using the appropriate routines
c
      if (ispot.eq.0) then
          call mopeg(v,x,dx,.true.,n3tm)
          if(lgs(25) .eq. 0) then
             if(icent.eq.0) call seccen(xii,fmat,mw,sstp)
             if(icent.eq.1) call seccep(mw,sstp,nend)
          else if(lgs(25) .eq. 1) then
                call deriv2(nend)                                       0307JC97
          else if(lgs(25) .eq. 2) then
                call derv24(nend)                                       0307JC97
          endif
          call mopeg(v,x,dx,.true.,n3tm)
      else if (ispot.eq.1) then
          call mopeg(v,x,dx,.true.,n3tm)
          call mophes(xii, fmat, n3tm)                                  1220WH94
c
c     for non-equilibrium solvation, need to add an extra harmonic term
c
            if (LBATH.AND.LFOPT) THEN
              nend = nend + 1
              F1 = PI*PI*REDM/(16.0d0*FRICT)
              F2 = F1/FRICT
              DO I = 1,NEND-1
                 FMAT(NEND,I) = -F2*DIFFU(I)
                 FMAT(I,NEND) = FMAT(NEND,I)
              ENDDO
              FMAT(NEND,NEND)=F2
              DO I = 1,NEND-1
               DO J = 1,I
                 FMAT(I,J) = FMAT(I,J)+F2*DIFFU(I)*DIFFU(J)
                 FMAT(J,I) = FMAT(I,J)
               ENDDO
              ENDDO
            endif
          do 100 i = 1, nend
             do 100 j = 1, i 
                fmat(i,j) = fmat(i,j) / (amass(ind(i))*amass(ind(j)))   0117WH95
                fmat(j,i) = fmat(i,j)
 100      continue
      end if
c
c     Put the energy at an appropriate zero of energy
      if (lezer0) v = v-ezer0
c
c
c   calculate the derivative and energy, should be in Cartesian coord
c
      IF (LBATH) THEN
        CALL EFFBATH(2)
      ENDIF
c
c
c     Transfer back to mass-weighted coordinates
      if (mw.eq.1) call trans(1,n3,amass,x,dx)
C
      return
C
      end                                           
